/*
-----------------------------------------------------------------------------
This source file is part of the GameTools Project
http://www.gametools.org

Author: Martin Szydlowski
-----------------------------------------------------------------------------
*/

#ifndef _OgreKdTreeSceneNode_H__
#define _OgreKdTreeSceneNode_H__

#include <OgreSceneNode.h>
#include "OgreKdRenderable.h"

namespace Ogre
{

	class KdTreeSceneNode : public SceneNode, public KdRenderable
	{
	public:
		KdTreeSceneNode(SceneManager *creator):
		  SceneNode(creator) {};

		KdTreeSceneNode(SceneManager *creator, const String& name):
		  SceneNode(creator, name) {};

		~KdTreeSceneNode() {};

		// calculate all-enclosing AABB, similar to _updateBounds()
		void forceComputeAABB(AxisAlignedBox& aabb); // OBSOLETE !!

		// gather info for kd-tree creation
		virtual void computeScene(PlaneEventList& events, AxisAlignedBox& aabb, int& nObjects, bool includeChildren = true);

		// add all contained objects to render queue
		virtual void queueObjects(Camera* cam, RenderQueue* queue, bool onlyShadowCasters);

		// place all entities in geometry queue (for CHC)
		virtual void getGeometryList(GtpVisibility::GeometryVector *geometryList);

		// return a bounding box enclosing all objects
		virtual AxisAlignedBox getBoundingBox() const;

		//virtual void _findVisibleObjects(Camera* cam, RenderQueue* queue, 
		//	bool includeChildren = true, bool displayNodes = false, bool onlyShadowCasters = false);

		// DEBUG
		String dumpToString();
	protected:
		virtual void _updateBounds(void);

		//AxisAlignedBox mLocalAABB;
	};

} // namespace Ogre

#endif // _OgreKdTreeSceneNode_H__